@@ -58,7 +58,7 @@ sf_project = function(from = character(0), to = character(0), pts, keep = FALSE,
 		stop("'keep' must be single-length non-NA logical value")
 	proj_from_crs = function(x) {
 		if (inherits(x, "crs")) {
-			x = if (sf_extSoftVersion()["proj.4"] >= "6.0.0")
+			x = if (compareVersion(sf_extSoftVersion()["proj.4"], "6.0.0") >= 0)
 				x$wkt
 			else
 				x$proj4string
@@ -203,7 +203,7 @@ print.proj_pipelines = function(x, ...) {
 	# nos:
 	if (length(nos) > 0L) {
 		grds <- attr(x, "grids")
-		for (i in seq(along=nos)) {
+		for (i in seq(along.with = nos)) {
 			grd <- grds[[nos[i]]]
 			ii <- length(grd)
 			if (ii > 0L) {
