@@ -26,8 +26,9 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' the same as specified by \code{size}? \code{TRUE} by default. Only applies to polygons, and
 #' when \code{type = "random"}.
 #' @param progress logical; if \code{TRUE} show progress bar (only if \code{size} is a vector).
+#' @param force logical; if `TRUE` continue when the sampled bounding box area is more than 1e4 times the area of interest, else (default) stop with an error. If this error is not justified, try setting `oriented=TRUE`, see details.
 #' @return an \code{sfc} object containing the sampled \code{POINT} geometries
-#' @details if \code{x} has dimension 2 (polygons) and geographical coordinates (long/lat), uniform random sampling on the sphere is applied, see e.g. \url{http://mathworld.wolfram.com/SpherePointPicking.html}. 
+#' @details if \code{x} has dimension 2 (polygons) and geographical coordinates (long/lat), uniform random sampling on the sphere is applied, see e.g. \url{https://mathworld.wolfram.com/SpherePointPicking.html}. 
 #'
 #' For \code{regular} or \code{hexagonal} sampling of polygons, the resulting size is only an approximation.
 #'
@@ -40,6 +41,8 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #'
 #' Sampling methods from package \code{spatstat} are interfaced (see examples), and need their own parameters to be set. 
 #' For instance, to use \code{spatstat.random::rThomas()}, set \code{type = "Thomas"}.
+#' 
+#' For sampling polygons one can specify `oriented=TRUE` to make sure that polygons larger than half the globe are not reverted, e.g. when specifying a polygon from a bounding box of a global dataset. The `st_sample` method for `bbox` does this by default.
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
 #' p1 = st_sample(nc[1:3, ], 6)
@@ -49,16 +52,16 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' plot(p2, add = TRUE, pch = 2)
 #' x = st_sfc(st_polygon(list(rbind(c(0,0),c(90,0),c(90,90),c(0,90),c(0,0)))), crs = st_crs(4326))
 #' plot(x, axes = TRUE, graticule = TRUE)
-#' if (sf_extSoftVersion()["proj.4"] >= "4.9.0")
+#' if (compareVersion(sf_extSoftVersion()["proj.4"], "4.9.0") >= 0)
 #'   plot(p <- st_sample(x, 1000), add = TRUE)
 #' if (require(lwgeom, quietly = TRUE)) { # for st_segmentize()
-#' x2 = st_transform(st_segmentize(x, 1e4), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
-#' g = st_transform(st_graticule(), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
-#' plot(x2, graticule = g)
-#' if (sf_extSoftVersion()["proj.4"] >= "4.9.0") {
-#'   p2 = st_transform(p, st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
-#'   plot(p2, add = TRUE)
-#' }
+#'   x2 = st_transform(st_segmentize(x, 1e4), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
+#'   g = st_transform(st_graticule(), st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
+#'   plot(x2, graticule = g)
+#'   if (compareVersion(sf_extSoftVersion()["proj.4"], "4.9.0") >= 0) {
+#'     p2 = st_transform(p, st_crs("+proj=ortho +lat_0=30 +lon_0=45"))
+#'     plot(p2, add = TRUE)
+#'   }
 #' }
 #' x = st_sfc(st_polygon(list(rbind(c(0,0),c(90,0),c(90,10),c(0,90),c(0,0))))) # NOT long/lat:
 #' plot(x)
@@ -69,7 +72,7 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' x = st_sfc(st_polygon(list(rbind(c(-180,-90),c(180,-90),c(180,90),c(-180,90),c(-180,-90)))),
 #'	 crs=st_crs(4326))
 #' # FIXME:
-#' #if (sf_extSoftVersion()["proj.4"] >= "4.9.0") {
+#' #if (compareVersion(sf_extSoftVersion()["proj.4"], "4.9.0") >= 0) {
 #' #  p = st_sample(x, 1000)
 #' #  st_sample(p, 3)
 #' #}
@@ -90,9 +93,9 @@ st_sample = function(x, size, ...) UseMethod("st_sample")
 #' plot(st_sample(ls, 80))
 #' # spatstat example:
 #' if (require(spatstat.random)) {
-#'  x <- sf::st_sfc(sf::st_polygon(list(rbind(c(0, 0), c(10, 0), c(10, 10), c(0, 0)))))
-#'  # for spatstat.random::rThomas(), set type = "Thomas":
-#'  pts <- st_sample(x, kappa = 1, mu = 10, scale = 0.1, type = "Thomas") 
+#'   x <- sf::st_sfc(sf::st_polygon(list(rbind(c(0, 0), c(10, 0), c(10, 10), c(0, 0)))))
+#'   # for spatstat.random::rThomas(), set type = "Thomas":
+#'   pts <- st_sample(x, kappa = 1, mu = 10, scale = 0.1, type = "Thomas") 
 #' }
 #' @export
 #' @name st_sample
@@ -101,7 +104,7 @@ st_sample.sf = function(x, size, ...) st_sample(st_geometry(x), size, ...)
 #' @export
 #' @name st_sample
 st_sample.sfc = function(x, size, ..., type = "random", exact = TRUE, warn_if_not_integer = TRUE,
-		by_polygon = FALSE, progress = FALSE) {
+		by_polygon = FALSE, progress = FALSE, force = FALSE) {
 
 	if (!missing(size) && warn_if_not_integer && any(size %% 1 != 0))
 		warning("size is not an integer")
@@ -118,7 +121,7 @@ st_sample.sfc = function(x, size, ..., type = "random", exact = TRUE, warn_if_no
 		res = switch(max(st_dimension(x)) + 1,
 					 st_multipoints_sample(do.call(c, x), size = size, ..., type = type),
 					 st_ll_sample(st_cast(x, "LINESTRING"), size = size, ..., type = type),
-					 st_poly_sample(x, size = size, ..., type = type, by_polygon = by_polygon))
+					 st_poly_sample(x, size = size, ..., type = type, by_polygon = by_polygon, force = force))
 		if (exact && type == "random" && all(st_geometry_type(res) == "POINT")) {
 			diff = size - length(res)
 			if (diff > 0) { # too few points
@@ -139,9 +142,34 @@ st_sample.sfg = function(x, size, ...) {
 	st_sample(st_geometry(x), size, ...)
 }
 
+#' @export
+#' @name st_sample
+#' @param great_circles logical; if `TRUE`, great circle arcs are used to connect the bounding box vertices, if `FALSE` parallels (graticules)
+#' @param segments units, or numeric (degrees); segment sizes for segmenting a bounding box polygon if `great_circles` is `FALSE`
+#' @examples
+#' bbox = st_bbox(
+#'	c(xmin = 0, xmax = 40, ymax = 70, ymin = 60),
+#' 	crs = st_crs('OGC:CRS84')
+#' )
+#' set.seed(13531)
+#' s1 = st_sample(bbox, 400)
+#' st_bbox(s1) # within bbox
+#' s2 = st_sample(bbox, 400, great_circles = TRUE)
+#' st_bbox(s2) # outside bbox
+st_sample.bbox = function(x, size, ..., great_circles = FALSE, segments = units::set_units(2, "degree", mode = "standard")) {
+	polygon = st_as_sfc(x)
+	crs = st_crs(x)
+	if (isTRUE(st_is_longlat(x)) && !great_circles) {
+		st_crs(polygon) = NA_crs_ # to fool segmentize that we're on R2:
+		segments = units::drop_units(units::set_units(segments, "degree", mode = "standard"))
+		polygon = st_set_crs(st_segmentize(polygon, segments), crs)
+	}
+	st_sample(polygon, size, ..., oriented = TRUE)
+}
+
 st_poly_sample = function(x, size, ..., type = "random",
-                          offset = st_sample(st_as_sfc(st_bbox(x)), 1)[[1]],
-						  by_polygon = FALSE) {
+						offset = st_sample(st_as_sfc(st_bbox(x)), 1)[[1]],
+						by_polygon = FALSE, oriented = FALSE, force = FALSE) {
 
 	if (by_polygon && inherits(x, "sfc_MULTIPOLYGON")) { # recurse into polygons:
 		sum_a = units::drop_units(sum(st_area(x)))
@@ -164,25 +192,36 @@ st_poly_sample = function(x, size, ..., type = "random",
 		global = FALSE
 		bb = st_bbox(x)
 		if (isTRUE(st_is_longlat(x))) {
-			if (sf_use_s2())
-				bb = st_bbox(st_segmentize(st_as_sfc(bb), 
-										   units::set_units(1, "degree", mode = "standard"))) # get coordinate range on S2
+			if (sf_use_s2()) { # if FALSE, the user wants the coord ranges to be the bbox
+				if (!requireNamespace("lwgeom", quietly = TRUE))
+					warning("coordinate ranges not computed along great circles; install package lwgeom to get rid of this warning")
+				else {
+					# see https://github.com/r-spatial/sf/issues/2331
+					# bb = st_bbox(st_segmentize(st_as_sfc(bb),
+					#		units::set_units(1, "degree", mode = "standard"))) # get coordinate range on S2
+					dfMaxLength = units::set_units(100000, "m", mode = "standard")
+					if (! is.na(st_crs(x)))
+						units(dfMaxLength) = units(st_crs(x)$SemiMajor) # might convert
+					seg = st_sfc(CPL_gdal_segmentize(x, dfMaxLength), crs = st_crs(x)) # avoid lwgeom path
+					bb = st_bbox(seg)
+				}
+			}
 			R = s2::s2_earth_radius_meters()
-			toRad = pi / 180
+			earth_surface = 4 * pi * R^2.
+			toRad = pi / 180.
 			h1 = sin(bb["ymax"] * toRad)
 			h2 = sin(bb["ymin"] * toRad)
-			a0 = 2 * pi * R^2. * (h1 - h2) * (bb["xmax"] - bb["xmin"]) / 360.
-			a1 = as.numeric(sum(st_area(x)))
+			a0 = 2 * pi * R^2. * (h1 - h2) * (bb["xmax"] - bb["xmin"]) / 360. # total
+			a1 = sum(s2::s2_area(st_as_s2(x, oriented = oriented))) # actual
 			if (!is.finite(a1))
 				stop("One or more geometries have a non-finite area")
-			if (a1 < 1) { # global polygon FIXME: 
-				a1 = a0
-				global = TRUE
-			}
+			global = (a0 / earth_surface) > .9999
+			if (a0 / a1 > 1e4 && !force)
+				stop(paste0("sampling box is ", format(a0/a1), " times larger than sampling region;\nuse force=TRUE if you really want this, or try setting oriented=TRUE\n(after reading the documentation)"), call. = FALSE)
 			size = round(size * a0 / a1)
 		} else {
-			a0 = as.numeric(st_area(st_as_sfc(bb)))
-			a1 = as.numeric(sum(st_area(x)))
+			a0 = as.numeric(st_area(st_as_sfc(bb))) # total
+			a1 = as.numeric(sum(st_area(x))) # actual
 			# we're sampling from a box, so n should be size_desired * a0 / a1
 			if (is.finite(a0) && is.finite(a1) && a0 > a0 * 0.0 && a1 > a1 * 0.0) { # FIXME: reqs can be removed, now we handle long/lat separately?
 				r = size * a0 / a1
@@ -192,6 +231,7 @@ st_poly_sample = function(x, size, ..., type = "random",
 						round(r)
 			}
 		}
+		size = max(1, size)
 
 		pts = if (type == "hexagonal") {
 				dx = sqrt(a0 / size / (sqrt(3)/2))
@@ -231,11 +271,14 @@ st_poly_sample = function(x, size, ..., type = "random",
 		spatstat_fun = try(get(paste0("r", type), asNamespace("spatstat.random")), silent = TRUE)
 		if (inherits(spatstat_fun, "try-error"))
 			stop(paste0("r", type), " is not an exported function from spatstat.random.")
-		pts = try(spatstat_fun(..., win = spatstat.geom::as.owin(x)), silent = TRUE)
+		pts = if ("win" %in% names(as.list(args(spatstat_fun))))
+				try(spatstat_fun(..., win = spatstat.geom::as.owin(x)), silent = TRUE)
+			else
+				try(spatstat_fun(..., W = spatstat.geom::as.owin(x)), silent = TRUE)
 		if (inherits(pts, "try-error"))
 			stop("The spatstat function ", paste0("r", type),
-             " did not return a valid result. Consult the help file.\n",
-             "Error message from spatstat:\n", pts)
+				" did not return a valid result. Consult the help file.\n",
+				"Error message from spatstat:\n", pts)
 		st_as_sf(pts)[-1,]
 	}
 }
